import json
import logging
try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode

import requests


class SSOAuthentication(object):

    def __init__(self, auth_url, api_key, callback_url, request_token_url, access_url, auth_token_url, error_handler, redirect_handler):
        self.api_key = api_key
        self.auth_url = auth_url
        self.callback_url = callback_url
        self.request_token_url = request_token_url
        self.access_url = access_url
        self.auth_token_url = auth_token_url
        self.redirect_handler = redirect_handler
        self.error_handler = error_handler

    def request_check(self, request_token, auth_token):
        have_access = False
        user = None
        if not (request_token and auth_token):
            return have_access, user

        try:
            res = requests.get(self.auth_token_url, params={
                'request_token': request_token,
                'auth_token': auth_token,
                'api_key': self.api_key
            }, verify=False)

            res.raise_for_status()
            response = json.loads(res.content.decode('utf-8'))
            if response['success'] and response['data']['user']:
                user = response['data']
                have_access = True
        except requests.HTTPError as e:
            if e.response.status_code >= 500:
                logging.exception(e)
        except Exception as e:
            logging.exception(e)

        return have_access, user

    def check_authentication(self, request_token=None, auth_token=None, user_id=None, redirect_next=None):
        have_access, user = self.request_check(request_token, auth_token)
        if not have_access:
            have_access, user = self.access_check(auth_token, user_id)

        if have_access and user:
            return user, True
        else:
            request_token = self.get_token()
            if not request_token:
                return self.error_handler(403, 'Forbidden, cause SSO server is not responding'), False

            params = {
                'request_token': request_token,
                'next': self.callback_url + redirect_next or '',
                'api_key': self.api_key,
            }
            redirect_url = '{0}?{1}'.format(
                self.auth_url,
                urlencode(params),
            )

            result = self.redirect_handler(redirect_url)

        return result, False

    def access_check(self, auth_token, user_id):
        have_access = False
        user = None
        if not (auth_token and user_id):
            return have_access, user

        try:
            res = requests.get(self.access_url, params={
                'auth_token': auth_token,
                'user_id': int(user_id),
                'api_key': self.api_key
            }, verify=False)
            res.raise_for_status()
            response = json.loads(res.content.decode('utf-8'))
            if response['success'] and response['data']['user']:
                user = response['data']
                have_access = True
        except requests.HTTPError as e:
            if e.response.status_code >= 500:
                logging.exception(e)
        except Exception as e:
            logging.exception(e)

        return have_access, user

    def get_token(self):
        request_token = None
        try:
            res = requests.get(self.request_token_url, params={
                'api_key': self.api_key
            }, verify=False)
            res.raise_for_status()
            response = json.loads(res.content.decode('utf-8'))
            if response['success'] and response['data']['request_token']:
                request_token = response['data']['request_token']
        except Exception as e:
            logging.exception(e)
        return request_token



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

VERSION = "0.0.19"

setup(
    name='sso-client-decorator',
    description='client decorator for sso authentication scheme',
    version=VERSION,
    url='https://github.com/KokocGroup/sso-client-decorator',
    download_url='https://github.com/KokocGroup/sso-client-decorator/tarball/v{0}'.format(VERSION),
    packages=find_packages(),
    package_dir={'sso_client_decorator': 'sso_client_decorator'},
    install_requires=[
        'requests'
    ],
)



from __future__ import absolute_import

from functools import wraps
import urllib
from flask import abort, current_app, redirect, request
from sso_client_decorator.sso import SSOAuthentication as BaseSSOAuthentication
from werkzeug.local import LocalProxy

try:
    from flask import _app_ctx_stack as stack
except ImportError:
    from flask import _request_ctx_stack as stack


class SSOAuthentication(object):

    def __init__(self, app=None):
        self.app = app
        self.sso = None
        self.config = None
        self.user_callback = None
        if self.app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.config = {
            'auth_url': app.config['SSO']['AUTH_URL'],
            'api_key': app.config['SSO']['API_KEY'],
            'callback_url': app.config['SSO']['CALLBACK_URL'],
            'request_token_url': app.config['SSO']['REQUEST_TOKEN_URL'],
            'access_url': app.config['SSO']['ACCESS_URL'],
            'auth_token_url': app.config['SSO']['AUTH_TOKEN_URL'],
            'redirect_handler': self.redirect_handler,
            'error_handler': self.error_handler
        }

        self.sso = BaseSSOAuthentication(**self.config)

        app.sso_manager = self
        app.context_processor(_user_context_processor)

    def redirect_handler(self, url):
        return redirect(url)

    def error_handler(self, status_code, msg):
        return abort(status_code)

    def user_loader(self, func):
        self.user_callback = func
        return func

    @property
    def user(self):
        return _get_user()

    def sso_access(self, func):
        @wraps(func)
        def wrapped(*args, **kwargs):
            request_token = request.args.get('request_token', request.cookies.get('request_token'))
            auth_token = request.args.get('auth_token', request.cookies.get('auth_token'))
            user_id = request.cookies.get('user_id')
            redirect_to = request.path
            if request.args:
                redirect_to = "{}?{}".format(redirect_to, urllib.urlencode(request.args))
            result, status = self.sso.check_authentication(request_token, auth_token, user_id, redirect_to)
            if status:
                ctx = stack.top
                if self.user_callback:
                    loaded_user = self.user_callback(result)
                    ctx.sso_user = loaded_user
                else:
                    ctx.sso_user = result
                view_result = func(*args, **kwargs)
                response = current_app.make_response(view_result)
                response.set_cookie('auth_token', auth_token)
                response.set_cookie('user_id', str(result['user']))
                response.set_cookie('request_token', request_token)
                return response
            return result
        return wrapped


def _user_context_processor():
    return dict(sso_user=_get_user())


def _get_user():
    return getattr(stack.top, 'user', None)

sso_user = LocalProxy(lambda: _get_user())
